
# q-expansion of newform 252.8.a.e, downloaded from the LMFDB on 14 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 252
weight = 8
poly_data = [-882, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-3, 6], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [0, 0], [-21, -1], [343, 0], [-3714, 14], [5915, 3], [-7896, -106], [13307, 9], [-16320, 504], [79008, 154], [-90370, 882], [-22912, -2562], [160860, 2714], [511934, -2058], [-832986, -998], [205314, -1344], [-851067, 7263], [-273763, -4863], [-1295308, -16968], [-2064636, -2716], [-4004434, 1836], [1235228, 41076], [4950393, -17693], [-7711746, 12056], [-8688736, -47790], [-6686925, -29097], [1021202, 84342], [-17039184, 37100], [-5363716, 72282], [-16084284, -51814], [-5247220, -194124], [-1377579, -65577], [-7353654, 22428], [-33208987, 132183], [-41648610, -81564], [-10072612, -74676], [31642835, -295941], [-1240018, 345870], [-70175238, 7326], [-18285183, 267809], [-40213992, -278516], [41297627, 26907], [61541244, -126364], [-41644648, 140826], [36225486, -83804], [70509278, -267414], [-115452292, -192864], [212910236, -153372], [-196966203, -676857], [83046215, 854559], [189161742, 716184], [91440480, -44240], [-129524248, 140730], [94666131, 1009953], [237601014, -1066808], [240155940, -1273468], [133070721, 1868929], [-198794680, 191400], [335701598, 1632960], [334905702, 1099952], [77543795, -1408311], [155462223, -203877], [-100274965, -1674183], [-39466224, 2027048], [405799604, -704442], [-380674026, -2381932], [297931502, -373338], [1013665136, 635250], [-3029238, -1529934], [-121218265, -4286649], [954609726, -238112], [-1124382876, 3012996], [711709124, -51348], [-559193086, -450660], [-338587990, 3894618], [-570004554, -6528294], [226030440, -10813894], [166157159, -123945], [518953992, -1342250], [-803935552, 3945402], [-1394225133, 674825], [-154883674, 5058984], [-1246200024, 7173544], [1535484860, 6229446], [608238008, -2609136], [-1396203156, -11084192], [-532414746, -16698472], [-178457872, 2216634], [-353148411, 592797], [-1734187744, -7965552], [-48658407, 5013379], [-2537995614, 3173934], [-2063441056, -920808], [3025150356, 7173544], [-2689337164, 6274632], [-1226922228, -8712116], [3322871097, -3248519], [-200138988, -17556286], [-1394753017, 17412453], [-1057733278, -12543468], [917086346, -17014830], [-434448786, -26046356], [-3956875713, 91725], [-461471400, 25804142], [-1433355790, -10422510], [-314927158, 26422776], [4011215271, -25554435], [7019564034, 6887468], [873566784, 14690200], [-388427326, 43024296], [2389745120, -12959040], [-2801207896, -28660842], [7024795464, -10031378], [-3332371021, 12255417], [5172995252, 6544692], [1462161504, 50420398], [-2676989119, 45996771], [2130330930, -5852602], [-1410595932, -52515610], [-749305506, -64928010], [2715028841, -40097667], [6429790262, -47336436], [6672308013, 5392525], [-5475986292, 22056048], [14279735939, -11398431], [-2238333156, 54171614], [9429070328, -31226202], [5985329322, 58028950], [-4946173750, -53826354], [874857221, 98687289], [-5979306226, 24905454], [3642260124, 21234444], [-7499966296, -26967864], [-1347007408, -4955874], [11266700424, -33010594], [-4444330492, -40838226], [-11074164969, -21718413], [-11843876065, -28186299], [-6139436919, 43304905], [-7329829656, -70339570], [10090767575, 48489909], [-27044161542, -560952], [-7740238144, 19432392], [14776464816, 39218620], [8828641157, 110630361], [-3027431946, -63044190], [11521615847, 36860175], [-21229008864, 70630310], [-20286754915, -77370801], [-3604580268, -140308756], [-26590847800, 8520750], [-7919434782, 5104940], [5215013456, -116188212], [24434792718, 30833594], [5890157384, -17563140], [-12890969580, 117587708], [-33264357568, 10175592], [19928444256, -21374546], [-5067179026, -42365172], [-24849508509, -43736849], [-24541389450, -27995842], [10760807502, 172592168], [7706420216, -89583648], [43369383981, -65191801], [-17159523534, -126829444], [-2214845178, 193140258], [13601142536, 68636568], [23853254705, 57347109], [23924615432, 73839738], [15829006641, -88634311], [-6920742300, -151983384], [-26601561301, -194050173], [18991165092, 19722500], [-5928390046, 278457816], [-43179411058, 4241202], [4495816290, 312024788], [4301027336, -39891348], [2856124968, 71941338], [-12230237572, -153348996], [21718472909, -107722911], [35200355912, 122193624], [78816114951, -34291499], [-15530389060, -81743550], [10213946526, -324346328], [30161784708, 13004236], [-4899532155, -155465947], [-21411911698, -97641432], [57844977659, 48452793], [16963034582, 164797668], [7811952918, 130598074], [58472431178, 163625016], [38602936068, -199539816], [57626188688, -21513492], [42070864941, 219984133], [-30030170520, -66503444], [16287896394, 197689276], [-63823100176, 196929810], [-43311140560, -53837322], [-331818942, 367196316], [32045369664, -490321720], [-38325233574, -49669312], [42075545162, 238218774], [-21630627823, 50177109], [-14040023410, -30110220], [17983269297, 505049603], [-45572535309, 49662255], [59547195722, -160588698], [-73128776718, 354056178], [-10209727956, 28384202], [3021834641, -239396085], [10778528864, -375989502], [3634447467, -22651633], [52292877596, -23792244], [-17658816147, 658056663], [-7704525024, -458223416], [-2779353304, 660139842], [-37054498288, -333138624], [22087777908, 124731114], [-18417920352, 132409144], [-60980024508, -323196650], [-170143787644, -63673638], [-53332690432, 414900936], [5727226326, -29056664], [-76063757200, 566651232], [27123365829, -924788297], [100651535108, 113296554], [73213400358, 720986736], [11610413304, 380134664], [57103672220, 35201820], [-22703089770, -1056227242], [-11101207108, 409351194], [-25833383611, -668351793], [-69434493928, 107430120], [33084355662, -179983048], [-29076894721, 901805997], [-67769143008, 679698160], [-41915256040, 176678634], [-29705539290, 553037660], [38298608652, 1026172896], [-83488565412, 996974508], [-156328650978, 281794646], [-86178079729, -779600211], [-76795689796, 739457052], [172470253142, -320680752], [127601475330, 600526636], [-95126016006, -845694002], [-87283171864, 117543480], [-75413705283, 64592975], [21074099198, -1108963338], [-154759310712, -489219416], [-183969395230, 695789556], [-46963374234, 1097218264], [132080912388, 183870372], [-91511441350, -100119936], [125332003587, -524325137], [110847956304, 828463412], [30631496684, 60201666], [242255803601, 556450875], [175507344747, -53245601], [-79346834728, 82023042], [-184016896576, -999264], [230144278074, 32468842], [-55372216123, -1594595631], [-78484491547, -1208220135], [175003683120, -382007394], [-97555889851, -1049746401], [-147180651426, 1279898172], [106773994446, -936457816], [171884078096, 912677892], [167526496620, -252411306], [-128298045505, 979881087], [99649675154, -154142310], [3063502694, -902413980], [-107172180460, 1736501004], [-52154591566, 1507170144], [272096590052, 162806916], [-115606477038, -878718134], [193074233906, 963785508], [223629343316, 86152374], [6966975393, -508623941], [320289790212, -717050180], [123883387784, 775453392], [-190780593402, 1043619522], [19015058153, -524942259], [-322758212161, -1025249043], [353747619588, -74645172], [-16897159126, 98502600], [115534533270, 1544708396], [-350494060966, 827128254], [-21033790722, -119708768], [5155937634, -328213480], [-49207079775, -2816023525], [53992052952, 1638374766], [-416974813143, -374676821], [-140609672146, -1104995976], [272179532961, 1954860145], [-257774673352, 1929340584], [-211206187227, 101991269], [36630830523, -1717060103], [-312341686447, -1265682093], [52182686696, -2812678110], [-199258519812, 444351054], [135764978456, -898679208], [-433205766942, 386167250], [431989675862, 1200224046], [-228142839226, 462687036], [52363875594, -701668030], [291756696329, 100967685], [486676429368, 169409632], [-332583671938, 2313539760], [-464400009612, 1386106044], [-116401326312, -1933182566], [178443510066, -1539817524], [-108019073074, 1570748718], [-196099796916, 1900027388], [145632144266, -1370194224], [-209455555953, -279045379], [19592725824, -35406336], [103672345406, -87659292], [-32953908714, -1139271560], [-449605942051, -997532073], [478327332158, 1219105356], [-19016031069, 1328472031], [30273621308, -1005700668], [184566434448, 2182147814], [-231853503364, -2146323258], [169567781402, -1342436382], [-210695684185, 328680693], [-165826702404, -2155796188], [-446835342564, -1527954106], [-352163583574, -1031633988], [307925905212, -1885736258], [-370591648966, 157518966], [71803547067, -748788223], [78331615112, 2836123836], [-408510843111, -482249525], [73047270116, 28445634], [-169711003938, -2501365056], [-186578972566, -2320982160], [150643347554, 441847326], [52582245242, 1692284580], [-498453314592, 488660326], [120635552541, 1274937853], [3403046420, -2562471996], [-637780174668, 371769062], [267868830006, 2372463534], [-466526778109, 1258092483], [331655556338, 2060973474], [574067341932, -1370542044], [-575153797785, -2650419501], [-214035994543, 2540254803], [-545250048766, 3238099704], [-651666699222, -2164455440], [-725096944054, -1708748010], [-3524752774, -1352979516], [538610777082, -1562804420], [268146373362, 2159731678], [511091342853, 289361807], [292811979534, -2925358240], [117650951736, 2793946400], [-279536233582, 220819452], [19656155106, 3103111044], [476316351276, 5093178468], [482104262088, -1071768572], [914460672419, 1974156945], [-627354970480, 1864747962], [-376294624671, -2352238527], [173455417952, -3490022088], [-406052742256, 2313009930], [380268128928, -5783922508], [496105105853, -4293301665], [222001931112, 442537480], [-231014722890, 277721836], [-69758356024, 5246547456], [103247981666, -513456972], [386059286781, -6364259137], [770669797968, 1339012976], [-837978537634, -2823389724], [1174399931208, -768219306], [-83604547264, -714883302], [604679759481, -3465264391], [85124180856, 2575740958], [-947893336432, -1297737000], [-135446791290, -3971460584], [1282267808465, -707161101], [-96005812728, -2330201760], [-799170813604, 4918858644], [538527984701, -1756099287], [528371904884, 31656576], [-1064147138250, 1432988826], [-287247969616, 5518528386], [-668348808813, -458927849], [105271627128, 1408386308], [534453893981, 5687516823], [174187193868, 8053013388], [1000177158470, 3969255108], [999718056470, 5434363674], [1002903568044, 2513557130], [-146183861512, -4954176276], [-888225358032, 6402093474], [-41087014339, -3590456223], [-27582555786, -913584504], [1007751841520, -3860200512], [-146750304288, 1068236390], [864288010761, -2483264111], [176439552002, 6192546822], [-961857089650, -4702046664], [-1194772366776, 2540333110], [38450854925, -1474119033], [524392532055, 498723269], [-175739776282, -1627123668], [-1493456393196, -219794946], [-346280507586, 2948814750], [-1025701400188, 4736119752], [-565621493350, 1264544064], [-295435071771, 6032989853], [-285802625508, -7167761020], [-241389056734, 963554118], [1342379686536, 5867564982], [1823365204332, -1884805300], [-667844489580, 5418995974], [419297291945, -653518227], [1732645043592, -485384088], [105115707543, 1481065597], [255767791412, -3094903818], [-1040268474441, 2810535131], [-854328296034, 2917250294], [-376565615730, -13565420344], [-321282292471, -13887685053], [-477309733782, 6508900030]]
